//
//  SMKishocho.m
//  KishochoApp
//
//  Created by Cyan K on 05/11/24.
//  cyanworks.net
//

#import "SMKishocho.h"

@implementation SMKishocho

#pragma mark Initialization and deallocation
- (id)init
{
	if (self = [super init]) {
		weatherData = [[NSMutableArray alloc] init];
		[self makeWebView];
	}
    return self;
}

- (void)dealloc
{
	[weatherData release];
	[webView release];
	[super dealloc];
}

#pragma mark Implementation
// Make instance of WebView (size 0x0)
- (void)makeWebView
{
	webView = [[WebView alloc] initWithFrame:NSMakeRect(0, 0, 0, 0)
				frameName:@"" groupName:@""];
	[webView setFrameLoadDelegate:self];
	[webView setPreferencesIdentifier:@"SoraMoyoWebView"];

	// Set preferences of WebView - Disable JavaScript and so on.
	WebPreferences	*webPref = [webView preferences];
	[webPref setJavaScriptEnabled:NO];
	[webPref setJavaEnabled:NO];
	[webPref setPlugInsEnabled:NO];
	[webPref setLoadsImagesAutomatically:NO];
    [webPref setDefaultTextEncodingName:@"shift_jis"];
}

// Start loading webpage
- (void)loadWeather:(NSString *)area withMode:(unsigned int)mode
			baseURI:(NSURL *)uri ifModSince:(NSString *)ims
{
	NSString	*path;

	// Initialize instance variables
	[weatherData removeAllObjects];
	[self setHttpStatus:0];
	[self setDownloadMode:mode];
	
	switch (mode) {
		case SM3daysWeather:
			path = [NSString stringWithFormat:@"jp/yoho/%@.html", area];
			break;
		case SM7daysWeather:
			path = [NSString stringWithFormat:@"jp/week/%@.html", area];
			break;
		default:
			NSLog(@"Invalied download mode: %@", mode);
			[self notifyConnectionError];
			break;
	}
	[self loadWebPage:[NSURL URLWithString:path relativeToURL:uri] ifModSince:ims];
}

- (void)loadWebPage:(NSURL *)uri ifModSince:(NSString *)ims
{
	NSMutableURLRequest	*urlRequest = [NSMutableURLRequest requestWithURL:uri
										cachePolicy:NSURLRequestReloadIgnoringCacheData
										timeoutInterval:59];

	#ifdef DEBUG
		NSLog(@"Load WebPage: %@", uri);
		NSLog(@"If-Modified-Since: %@", ims);
	#endif
	// Set If-Modified-Since
	if (0 < [ims length])
		[urlRequest setValue:ims forHTTPHeaderField:@"If-Modified-Since"];

	[[webView mainFrame] loadRequest:urlRequest];
}

- (void)stopLoading
{
	#ifdef DEBUG
		NSLog(@"stopLoading");
	#endif
	[[webView mainFrame] stopLoading];
}

// Read weather data from HTML
- (void)readDocument:(WebFrame *)frame
{
	SMKishochoParser	*kishochoParser = [[[SMKishochoParser alloc] init] autorelease];
	NSArray				*parsedData;

	#ifdef DEBUG
		NSLog(@"Reading the document.");
	#endif
	parsedData = [kishochoParser parseDocument:[frame DOMDocument]
					withMode:[self downloadMode]];
	if (parsedData) {
		[weatherData addObjectsFromArray:parsedData];
		[self notifyDownloadFinished];
		#ifdef DEBUG
			int	i;
			for (i=0; i<[weatherData count]; i++)
				NSLog (@"%d:%@", i, [weatherData objectAtIndex:i]);
		#endif
	} else {
		[self notifyParseError];
	}
}

#pragma mark Notification
- (void)notifyDownloadFinished
{
	[[NSNotificationCenter defaultCenter]
		postNotificationName:@"weatherDownloadFinished" object:self userInfo:nil];
}

- (void)notifyConnectionError
{
	[[NSNotificationCenter defaultCenter]
		postNotificationName:@"connectionError" object:self userInfo:nil];
}

- (void)notifyParseError
{
	[[NSNotificationCenter defaultCenter]
		postNotificationName:@"parseError" object:self userInfo:nil];
}

#pragma mark Accessor methods
- (int)httpStatus
{
	return httpStatus;
}
- (void)setHttpStatus:(int)newHttpStatus
{
	httpStatus = newHttpStatus;
}

- (int)downloadMode
{
	return downloadMode;
}
- (void)setDownloadMode:(int)newDownloadMode
{
	downloadMode = newDownloadMode;
}

- (NSArray *)weatherData
{
	return weatherData;
}

#pragma mark WebView delegate
// WebFrame loading finished
- (void)webView:(WebView *)sender didFinishLoadForFrame:(WebFrame *)frame
{
	if ([sender mainFrame] != frame) return;
	NSHTTPURLResponse	*HUResponse = (NSHTTPURLResponse *)[[frame dataSource] response];
	NSDictionary		*dicHead = [HUResponse allHeaderFields];
	NSString			*lastModified = [dicHead objectForKey:@"Last-Modified"];
	[self setHttpStatus:[HUResponse statusCode]];

	// Print HTTP response header
	#ifdef DEBUG
		NSLog (@"didFinishLoadForFrame");
		NSEnumerator	*enumHead = [dicHead keyEnumerator];
		NSMutableString	*sHead = [NSMutableString string];
		NSString		*sValue;
		id key;

		if (lastModified)
			NSLog(@"lastModified: %@", lastModified);
		while ((key = [enumHead nextObject])) {
			sValue = [dicHead objectForKey:key]; 
			[sHead appendFormat:@"%@ = %@", key, sValue];
			[sHead appendFormat:@"%C", 0x000D];
		}
		NSLog(@"HEADER: %@", sHead);
		NSLog(@"MIMEType: %@", [HUResponse MIMEType]);
		NSLog(@"textEncodingName: %@", [HUResponse textEncodingName]);
		NSLog(@"statusCode: %d", [self httpStatus]);
	#endif

	switch ([self httpStatus]) {
		case 200:	// Success
			if (lastModified)
				[weatherData addObject:lastModified];
			else
				[weatherData addObject:@"No Last-Modified"];
			[self readDocument:frame];
			break;
		case 304:	// Not modified
			[self notifyDownloadFinished];
			break;
		default:	// Error
			[self notifyDownloadFinished];
			break;
	}
}

// WebFrame loading error
- (void)webView:(WebView *)sender didFailLoadWithError:(NSError *)error
	forFrame:(WebFrame *)frame
{
	if ([sender mainFrame] != frame) return;

	NSLog(@"Connection failed! didFailLoadWithError - %@ %@",
		[error localizedDescription],
		[[error userInfo] objectForKey:NSErrorFailingURLStringKey]);
	[self notifyConnectionError];
}

- (void)webView:(WebView *)sender didFailProvisionalLoadWithError:(NSError *)error
	forFrame:(WebFrame *)frame
{
	if ([sender mainFrame] != frame) return;

	NSLog(@"Connection failed! didFailProvisionalLoadWithError - %@ %@",
		[error localizedDescription],
		[[error userInfo] objectForKey:NSErrorFailingURLStringKey]);
	[self notifyConnectionError];
}

@end
